module RechnMgmUserArea
    class ApplicationController < ActionController::Base
        protect_from_forgery with: :exception

        NotAuthorised = Class.new(StandardError)

        rescue_from RechnMgmUserArea::ApplicationController::NotAuthorised do
            render_error_page(status: 403, text: 'Forbidden')
        end

        before_action :add_x_frame_options if Rails.env.development?
        before_action :set_membership
        before_action :set_locale

        private
        def add_x_frame_options
            response.headers['X-FRAME-OPTIONS'] = 'ALLOWALL'
        end

        def set_membership
            # set membership if organisation was set by the token
            raise RechnMgmUserArea::ApplicationController::NotAuthorised if current_user.nil?

            begin
                token = Warden::JWTAuth::HeaderParser.from_env(request.headers)
                if token.nil?
                    # Seems that we were called via a session based authentication. Thus we need to set the current membership..
                    current_user.init_current_membership if current_user.current_membership_id.nil?
                    return;
                end

                payload = Warden::JWTAuth::TokenDecoder.new.call(token)
                current_user.current_membership_id = payload["membership_id"]
            rescue Exception => e
                Rails.logger.error "Token error: #{e}"
                raise RechnMgmUserArea::ApplicationController::NotAuthorised
            end
        end

        def render_error_page(status:, text:, template: 'errors/routing')
            respond_to do |format|
                format.json { render json: {errors: [message: "#{status} #{text}"]}, status: status }
                format.html { redirect_to "/logout" }
                format.any  { head status }
            end
        end


        def set_locale
            # TODO: Set time zone in User profile.
            Time.zone = "Europe/Berlin"
            # Use locale from browser of default as fallback
            begin
                I18n.locale = extract_locale_from_accept_language_header
            rescue I18n::InvalidLocale => e
                EventLogger.log_system( "Setting Locale", "Unable to set locale. Reason: #{e}", type: EventLog::EVENT_LOG_TYPE_ERROR )
            end
        end

        def extract_locale_from_accept_language_header
            if ( request.env['HTTP_ACCEPT_LANGUAGE'].nil? )
                return I18n.default_locale
            end
            request.env['HTTP_ACCEPT_LANGUAGE'].scan(/^[a-z]{2}/).first
        end

    end
end
